
Global _g_ib3d_collision_manager:COLLISION_MANAGER

New COLLISION_MANAGER

' Collision Detection Methods
Const CD_SPHERE_SPHERE_IRRLICHT:Int=1
Const CD_ELLIPSOID_POLY_IRRLICHT:Int=2
Const CD_ELLIPSOID_POLY_STATIC_IRRLICHT:Int=3
Const CD_ELLIPSOID_BOX_IRRLICHT:Int=4 ' use getTransformedBoundingbox()?
Const CD_BOX_BOX_IRRLICHT:Int=5 ' use aabbox3d->intersectsWithBox()

' Collision Response
Const CR_NOTHING:Int=0
Const CR_STOP:Int=1
Const CR_SLIDE_FULL_IRRLICHT:Int=2 ' slides on all axis (full slide)
Const CR_SLIDE_LIMITED_IRRLICHT:Int=3 ' slides just on UP, X & Z axis

Type COLLISION_MANAGER
	Field _collision_map:TMap
	Field _collision_types:TMap
	Field _cnt:Int=0
	Field _typecnt:Int=0
	
	Method New()
		_g_ib3d_collision_manager=Self
	EndMethod
	
	Method detectCollisions(bRespond:Int=True)		
		If _collision_map
			For Local map:COLLISION_MAP=EachIn _collision_map.Values()
				map.checkForCollisions(bRespond)
			Next
			' set the last position.  this needs to happen here in order to avoid subsequent checks to not firing.
			For Local ctype:COLLISIONTYPE=EachIn _collision_types.Values()
				If ctype
					For Local entity:ib3d_Entity=EachIn ctype._entities
						entity.setLastPosition()
					Next
				EndIf
			Next
		EndIf
	EndMethod

	' bCountSourceOnly is true to return only the collisions where the passed entity is the src entity
	Method countCollisions:Int(entity:ib3d_Entity,bCountSourceOnly:Int=True)
		Local retval:Int=0
		If _collision_map Then 
			For Local map:COLLISION_MAP = EachIn _collision_map.Values()			
				' first make sure there are collisions on this one
				retval:+map.getEntityCollisionCount(entity,bCountSourceOnly)
			Next
		EndIf
		Return retval
	EndMethod

	' bSourceOnly limits to only if entity is src entity
	Method getCollisionList:TList(entity:ib3d_Entity,bSourceOnly:Int=True)
		Local collisions:TList=Null
		If _collision_map Then 
			For Local map:COLLISION_MAP = EachIn _collision_map.Values()
				' first make sure there are collisions on this one
				Local maplist:TList=map.getEntityCollisionList(entity,bSourceOnly)				
				If maplist Then
					If Not collisions Then collisions=New TList
					For Local collision:COLLISION = EachIn maplist
						collisions.AddLast(collision)
					Next
				EndIf
			Next
		EndIf				
		Return collisions
	EndMethod

	Method collidedWith:Int(entity:ib3d_Entity,other_type:Int,bSourceOnly:Int=True)
		Local collisions:TList=getCollisionList(entity)
		If collisions
			For Local collision:COLLISION = EachIn collisions
				If collision._entity=entity And collision._collideWith_type=other_type Then Return True
				If Not bSourceOnly And collision._collideWith=entity And collision._entity_type=other_type Then Return True
			Next
		EndIf
		Return False
	EndMethod

	Method collisionTime:Long(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
		Local collision:COLLISION=getCollision(entity,idx,bSourceOnly)
		If collision Return collision.getTime() Else Return 0
	EndMethod
	
	Method getCollision:COLLISION(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
		Local collisions:TList=getCollisionList(entity,bSourceOnly)
		If collisions And (idx>=0 And idx<collisions.Count()) Then Return COLLISION(collisions.ToArray()[idx]) Else Return Null
	EndMethod

	' ----------------------------------------------------------
	' generation routines
	' ----------------------------------------------------------

	Method generateCollisionMap:COLLISION_MAP()
		Return New COLLISION_MAP
	EndMethod
		
	Method generateCollisionType:COLLISIONTYPE()
		Return New COLLISIONTYPE
	EndMethod
	
	Method generateCollision:COLLISION()
		Return New COLLISION
	EndMethod

	Method generateDetectionMethod:ICOLLISION_DETECT(collision_method:Int)
		Local retval:ICOLLISION_DETECT=Null
		
		Select collision_method
			Case CD_SPHERE_SPHERE_IRRLICHT
				retval=DETECT_SPHERE_SPHERE_IRRLICHT.create()
			Case CD_ELLIPSOID_POLY_IRRLICHT
				retval=DETECT_ELLIPSOID_POLY_IRRLICHT.create()
			Case CD_ELLIPSOID_POLY_STATIC_IRRLICHT
				retval=DETECT_ELLIPSOID_POLY_STATIC_IRRLICHT.create()
			Case CD_ELLIPSOID_BOX_IRRLICHT
				retval=DETECT_ELLIPSOID_BOX_IRRLICHT.create()
			Case CD_BOX_BOX_IRRLICHT
				retval=DETECT_BOX_BOX_IRRLICHT.create()
		EndSelect
		
		Return retval
	EndMethod

	Method generateResponseType:ICOLLISION_RESPONSE(collision_response:Int)
		Local retval:ICOLLISION_RESPONSE=Null
		
		' create a collision response instance
		Select collision_response
			Case CR_NOTHING
			Case CR_STOP
				retval=New RESPONSE_STOP
			Case CR_SLIDE_FULL_IRRLICHT
				retval=New RESPONSE_SLIDE_FULL_IRRLICHT
			Case CR_SLIDE_LIMITED_IRRLICHT
				retval=New RESPONSE_SLIDE_LIMITED_IRRLICHT
		EndSelect		
		
		Return retval
	EndMethod

	' ----------------------------------------------------
	' type mappings
	' ----------------------------------------------------

	Method addTypeMapping:COLLISION_MAP(src_type:Int,dest_type:Int,col_method:Int,col_response:Int,col_name:String="")
		' create a collision detection instance and response type instance and create the map
		Return addCustomTypeMapping(src_type,dest_type,generateDetectionMethod(col_method),generateResponseType(col_response),col_name)
	EndMethod
		
	Method addCustomTypeMapping:COLLISION_MAP(src:Int,dest:Int,det_meth:ICOLLISION_DETECT,response:ICOLLISION_RESPONSE=Null,col_name:String="")
		If Not _collision_map Then _collision_map=New TMap
		If col_name.length=0 Then col_name=_cnt
		If _collision_map.ValueForKey(col_name) Then RuntimeError "Collision "+col_name+" already exists"
		
		Local col:COLLISION_MAP=generateCollisionMap()	
		' add the collision types if they dont exist
		col.set(addType(src),addType(dest),det_meth,response)
		_collision_map.Insert(col_name,col)
		Return col
	EndMethod
	
	Method clearTypeMap()
		If _collision_map Then _collision_map.clear()
		_cnt=0
	EndMethod
	
	Method removeTypeMapping(col_name:String)
		If _collision_map And _collision_map.Remove(col_name) Then _cnt:-1
	EndMethod

	Method getTypeMappingCount:Int()
		Return _cnt
	EndMethod

	Method getTypeMapping:COLLISION_MAP(col_name:String)
		If _collision_map Then Return COLLISION_MAP(_collision_map.ValueForKey(col_name)) Else Return Null
	EndMethod

	Method getTypeMappingByIdx:COLLISION_MAP(col_idx:Int)
		If _collision_map Then
			Local cnt:Int=0
			For Local map:COLLISION_MAP = EachIn _collision_map.Values()
				If cnt=col_idx Then Return map
				cnt:+1
			Next
		Else
			Return Null
		EndIf
	EndMethod

	' ----------------------------------------------------
	' entity collision types
	' ----------------------------------------------------
	
	Method clearTypes()
		If _collision_types Then _collision_types.clear()
		_typecnt=0
	EndMethod
			
	Method removeType(collision_type:Int)
		If _collision_types And _collision_types.ValueForKey(String.FromInt(collision_type))
			_collision_types.Remove(String.FromInt(collision_type))
			_typecnt:-1
		EndIf
	EndMethod
	
	Method addType:COLLISIONTYPE(collision_type:Int)
		If Not _collision_types Then _collision_types=New TMap
		Local retval:COLLISIONTYPE=COLLISIONTYPE(_collision_types.ValueForKey(String.FromInt(collision_type)))
		If Not retval
			retval=generateCollisionType()
			retval._type=collision_type
			_collision_types.Insert(String.FromInt(collision_type),retval)
			_typecnt:+1
		EndIf
		Return retval
	EndMethod
	
	Method getType:COLLISIONTYPE(collision_type:Int)
		If _collision_types Then Return COLLISIONTYPE(_collision_types.ValueForKey(String.FromInt(collision_type))) Else Return Null
	EndMethod

	Method getTypeByIdx:COLLISIONTYPE(type_idx:Int)
		If _collision_types Then
			Local cnt:Int=0
			For Local col_type:COLLISIONTYPE = EachIn _collision_types.Values()
				If cnt=type_idx Then Return col_type
				cnt:+1
			Next
			Return Null ' didnt find it
		Else
			Return Null ' not types defined
		EndIf
	EndMethod
	
	Method getTypeCount:Int()
		Return _typecnt
	EndMethod
	
	' ----------------------------------------------------
	' entity mappings
	' ----------------------------------------------------

	Method addEntityToType(entity:ib3d_Entity,collision_type:Int,bChildren:Int=False)
		addType(collision_type) ' add the type if it doesnt exist
		Local col_type:COLLISIONTYPE=COLLISIONTYPE(_collision_types.ValueForKey(String.FromInt(collision_type)))
		If Not col_type.containsEntity(entity) Then col_type.addEntity(entity,bChildren)
	EndMethod
	
	Method removeEntityFromType(entity:ib3d_Entity,collision_type:Int,bChildren:Int=False)
		Local col_type:COLLISIONTYPE=COLLISIONTYPE(_collision_types.ValueForKey(String.FromInt(collision_type)))
		If col_type Then col_type.RemoveEntity(entity,bChildren)
	EndMethod
	
	Method removeEntityFromAllTypes(entity:ib3d_Entity,bChildren:Int=False)
		For Local col_type:COLLISIONTYPE = EachIn _collision_types.Values()
			col_type.removeEntity(entity,bChildren)
		Next
	EndMethod
	
	' returns the # of types the entity belongs to
	Method getEntityTypeCount:Int(entity:ib3d_Entity)
		Local retval:Int=0
		If _collision_types Then
			For Local col_type:COLLISIONTYPE = EachIn _collision_types.Values()
				If col_type.containsEntity(entity) Then retval:+1
			Next
		EndIf
		Return retval
	EndMethod
	
	' returns the type based on index
	Method getEntityTypeByIdx:COLLISIONTYPE(entity:ib3d_Entity,idx:Int=0)
		If Not _collision_types Then RuntimeError "No collision types have been defined"
		If (idx<0 Or idx>=getEntityTypeCount(entity)) Then RuntimeError "Collision type index "+idx+" out of range"
		Local cnt:Int=0
		For Local col_type:COLLISIONTYPE = EachIn _collision_types.Values()
			If col_type.containsEntity(entity) Then
				If cnt=idx Then Return col_type Else cnt:+1
			EndIf
		Next
		Return Null
	EndMethod
	
EndType

Type ONCOLLISIONEVENTHANDLER
	Method OnCollision(collision:COLLISION) Abstract
EndType

Type COLLISIONTYPE
	Field _type:Int
	Field _entities:TList
	Field _OnCollision:ONCOLLISIONEVENTHANDLER=Null
	
	Method setOnCollisionEventHandler(handler:ONCOLLISIONEVENTHANDLER)
		_OnCollision=handler
	EndMethod
	
	Method OnCollision(collision:COLLISION)
		If _OnCollision Then _OnCollision.OnCollision(collision)
	EndMethod

	Method getType:Int()
		Return _type
	EndMethod

	Method clearEntities()
		If _entities Then
			While getEntityCount()>0 
				removeEntity(getEntity(getEntityCount()-1))
			Wend
		EndIf
		_entities.Clear()
	EndMethod

	Method getEntityCount:Int()
		If _entities Then Return _entities.Count() Else Return 0
	EndMethod

	Method getEntity:ib3d_Entity(idx:Int)
		If _entities And (idx>=0 And idx<getEntityCount()) Then Return ib3d_Entity(_entities.ToArray()[idx]) Else Return Null
	EndMethod
									
	Method removeEntity(entity:ib3d_Entity,bChildren:Int=False)
		If _entities
			' remove all children if requested
			If bChildren
				Local childcnt:Int=entity._node.getChildren().Dimensions()[0]
				For Local i:Int=0 To childcnt
					Local child:ib3d_Entity=ib3d_Entity.findEntity(entity._node.getChild(i))
					If child Then removeEntity(child,True)
				Next			
			EndIf
		
			_entities.Remove(entity)
		EndIf
	EndMethod
	
	Method addEntity(entity:ib3d_Entity,bChildren:Int=False)
		If Not _entities Then _entities=New TList
		_entities.AddLast(entity)
		entity.setLastPosition()
		' add the children if requested
		If bChildren
			Local childcnt:Int=entity._node.getChildren().Dimensions()[0]
			For Local i:Int=0 To childcnt
				Local child:ib3d_Entity=ib3d_Entity.findEntity(entity._node.getChild(i))
				If child Then addEntity(child,True)
			Next
		EndIf		
	EndMethod
	
	Method containsEntity:Int(entity:ib3d_Entity)
		If Not _entities Return False Else Return _entities.contains(entity)
	EndMethod
EndType

' fires when the the collision method is set.  this can be useful in setting up collision information
' for the entities contained within.
Type ONSETCOLLISIONMETHODHANDLER
	Method OnSetCollisionMethod(collision_map:COLLISION_MAP) Abstract
EndType

Type COLLISION_MAP
	Field _src:COLLISIONTYPE
	Field _dest:COLLISIONTYPE
	Field _method:ICOLLISION_DETECT
	Field _OnSetMethod:ONSETCOLLISIONMETHODHANDLER=Null
	Field _bIncludeInCounts:Int=True
	
	Field _collisions:TList
	
	Method set(src:COLLISIONTYPE,dest:COLLISIONTYPE,meth:ICOLLISION_DETECT,resp:ICOLLISION_RESPONSE=Null)
		_src=src
		_dest=dest
		setCollisionMethod(meth)
		setCollisionResponse(resp)
	EndMethod
	
	Method setIncludeInCounts(bInclude:Int=True)
		_bIncludeInCounts=bInclude
	EndMethod
				
	Method getCollisionMethodType:Int()
		If _method Then Return _method._method Else Return -1
	EndMethod
	
	Method getCollisionMethod:ICOLLISION_DETECT()
		Return _method
	EndMethod

	Method setCollisionMethod(meth:ICOLLISION_DETECT)
		_method=meth
		' call the onset method in set to allow initialization of entity collision info if needed
		If _OnSetMethod Then _OnSetMethod.OnSetCollisionMethod(Self)
	EndMethod
		
	Method setOnCollisionMethodHandler(handler:ONSETCOLLISIONMETHODHANDLER)
		_OnSetMethod=handler
		' if a method already exists, then call the event
		If _method And _OnSetMethod Then _OnSetMethod.OnSetCollisionMethod(Self)
	EndMethod
		
	Method getCollisionResponseType:Int()
		If _method._response Then Return _method._response._response Else Return CR_NOTHING
	EndMethod
	
	Method getCollisionResponse:ICOLLISION_RESPONSE()
		Return _method._response
	EndMethod
	
	Method setCollisionResponse(resp:ICOLLISION_RESPONSE)
		If Not _method Then RuntimeError "Detection method must be set before response"
		_method.setCollisionResponse(resp)
	EndMethod
	
	Method getCollisionCount:Int()
		If _collisions Then Return _collisions.Count() Else Return 0
	EndMethod

	' bSourceOnly is true to only include collisions where the entity is the src entity
	Method getEntityCollisionList:TList(entity:ib3d_Entity,bSourceOnly:Int=True)
		If Not entity Then RuntimeError "Entity is not valid"
		Local retval:TList=Null
		Local collision_cnt:Int=getCollisionCount()
		If collision_cnt>0
			' check each collision looking for the entity
			For Local i:Int=0 To collision_cnt-1
				Local collision:COLLISION=getCollision(i)
				If collision._entity=entity Then
					If Not retval Then retval=New TList
					retval.AddLast(collision)
				ElseIf Not bSourceOnly And collision._collideWith=entity Then
					If Not retval Then retval=New TList
					retval.AddLast(collision)
				EndIf
			Next
		EndIf		
		Return retval
	EndMethod

	Method getEntityCollisionCount:Int(entity:ib3d_Entity,bSourceOnly:Int=True)
		If Not _bIncludeInCounts Then Return 0
		Local collisions:TList=getEntityCollisionList(entity,bSourceOnly)
		If collisions Then Return collisions.Count() Else Return 0
	EndMethod

	Method getEntityCollision:COLLISION(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
		Local collisions:TList=getEntityCollisionList(entity,bSourceOnly)
		If collisions And (idx>=0 And idx<collisions.Count()) Then Return COLLISION(collisions.ToArray()[idx]) Else Return Null
	EndMethod
									
	Method getCollision:COLLISION(idx:Int)
		If _collisions And (idx>=0 And idx<_collisions.Count()) Then Return COLLISION(_collisions.ToArray()[idx]) Else Return Null
	EndMethod
	
	Method clearCollisions()
		If _collisions Then _collisions.Clear()
	EndMethod
	
	Method checkForCollisions:Int(bRespond:Int=True)
		If Not _collisions Then _collisions=New TList Else clearCollisions()
	
		Local cnt:Int=0
	
		' if we have a method to check with, perform the check
		If _method And _src And _src._entities And _dest And _dest._entities			
			For Local entity:ib3d_Entity=EachIn _src._entities
				For Local other:ib3d_Entity=EachIn _dest._entities
					' skip if the entities are the same
					If entity._node.handle=other._node.handle Then Continue  
					
					Local time:Long=MilliSecs()
					Local col:COLLISION=_method.GetCollision(entity,other)
					If col
						' if the time was already set by the detection method, then skip this
						If col._time=0 Then col._time=MilliSecs()-time
						col._entity_type=_src.getType()
						col._collideWith_type=_dest.getType()

						_collisions.AddLast(col)
												
						col.setResponse(_method._response)
						If bRespond Then col.Respond()
						
						' fire the events for the entities
						col._entity.OnCollision(col)
						col._collideWith.OnCollision(col)
						
						' fire the events for the entity collision types
						_src.OnCollision(col)
						_dest.OnCollision(col)
						
						cnt:+1
					EndIf
				Next
			Next			
		EndIf
		
		Return cnt
	EndMethod	
EndType

Type COLLISION
	Field _entity:ib3d_Entity
	Field _entity_type:Int
	Field _collideWith:ib3d_Entity
	Field _collideWith_type:Int
	Field _method:Int
	Field _response:ICOLLISION_RESPONSE
	Field _triangle:Triangle3df
	Field _position:Vector3df
	Field _falling:Int
	Field _point:Vector3df
	Field _time:Long
	
	Method getTriangle:Triangle3df()
		If _triangle And _triangle.IsValid() Then Return _triangle Else Return Null
	EndMethod
	
	Method getNewPosition:Vector3df()
		Return _position
	EndMethod
	
	Method isFalling:Int()
		Return _falling
	EndMethod
	
	Method getTime:Long()
		Return _time
	EndMethod
	
	Method getPoint:Vector3df()
		If _point And _point.IsValid() Then Return _point Else Return Null
	EndMethod
	
	Method getSrcEntity:ib3d_Entity()
		Return _entity
	EndMethod

	Method getSrcEntityType:Int()
		Return _entity_type
	EndMethod

	Method getDestEntity:ib3d_Entity()
		Return _collideWith
	EndMethod

	Method getDestEntityType:Int()
		Return _collideWith_type
	EndMethod
								
	Function create:COLLISION(entity:ib3d_Entity,with:ib3d_Entity,collision_method:Int,tri:Triangle3df=Null,pos:Vector3df=Null,bFalling:Int=False,point:Vector3df=Null,time:Long=0)
		Local retval:COLLISION=_g_ib3d_collision_manager.generateCollision()
		retval._entity=entity
		retval._collideWith=with
		retval._method=collision_method
		retval._triangle=tri
		retval._position=pos
		retval._falling=bFalling
		retval._point=point
		retval._time=time
		Return retval
	EndFunction

	Method setResponse(response:ICOLLISION_RESPONSE=Null)
		_response=response
	EndMethod
			
	Method Respond()
		If _response Then _response.OnCollision(Self)
	EndMethod
EndType

'-----------------------------------------
' collision management functions
'-----------------------------------------

' TODO: note new params for name and gravity
Function ib3d_Collisions:COLLISION_MAP(src_type:Int,dest_type:Int,col_method:Int,col_response:Int,collision_name:String="")
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.addTypeMapping(src_type,dest_type,col_method,col_response,collision_name)
EndFunction

Function ib3d_ClearCollisions()
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	_g_ib3d_collision_manager.clearTypeMap()
EndFunction

' TODO: note new function to remove single collision mapping
Function ib3d_RemoveCollisionMap(collision_name:String)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	_g_ib3d_collision_manager.removeTypeMapping(collision_name)
EndFunction

' exclude allows skipping of polystatic collisions in the count
Function ib3d_CountCollisions:Int(entity:ib3d_Entity,bSourceOnly:Int=True)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.countCollisions(entity,bSourceOnly)
EndFunction

Function ib3d_GetCollisionList:TList(entity:ib3d_Entity,bSourceOnly:Int=True)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.getCollisionList(entity,bSourceOnly)	
EndFunction

Function ib3d_CollisionTime:Long(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then RuntimeError "Entity is not valid"
	Return _g_ib3d_collision_manager.collisionTime(entity,idx,bSourceOnly)
EndFunction

Function ib3d_CollisionEntity:ib3d_Entity(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then RuntimeError "Entity is not valid"
	Local collision:COLLISION=_g_ib3d_collision_manager.getCollision(entity,idx,bSourceOnly)
	If Not collision Return Null
	If collision.getSrcEntity()=entity Then Return collision.getSrcEntity() Else Return collision.getDestEntity()
EndFunction

Function ib3d_CollisionPoint:Vector3df(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then RuntimeError "Entity is not valid"
	Local collision:COLLISION=_g_ib3d_collision_manager.getCollision(entity,idx,bSourceOnly)
	If Not collision Then RuntimeError "Invalid collision index"
	Return collision.getPoint()
EndFunction

Function ib3d_CollisionX:Float(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	Local point:Vector3df=ib3d_CollisionPoint(entity,idx,bSourceOnly)
	If point Then Return point.getX() Else Return 0
EndFunction

Function ib3d_CollisionY:Float(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	Local point:Vector3df=ib3d_CollisionPoint(entity,idx,bSourceOnly)
	If point Then Return point.getX() Else Return 0
EndFunction

Function ib3d_CollisionZ:Float(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	Local point:Vector3df=ib3d_CollisionPoint(entity,idx,bSourceOnly)
	If point Then Return point.getX() Else Return 0
EndFunction

' TODO: note this returns the actual triangle now
Function ib3d_CollisionTriangle:Triangle3df(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then RuntimeError "Entity is not valid"
	Local collision:COLLISION=_g_ib3d_collision_manager.getCollision(entity,idx,bSourceOnly)
	If Not collision Then RuntimeError "Invalid collision index"
	Return collision.getTriangle()
EndFunction

Function ib3d_CollisionNX:Float(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	Local tri:Triangle3df=ib3d_CollisionTriangle(entity,idx,bSourceOnly)
	If tri Then Return tri.getNormal().getX()
EndFunction

Function ib3d_CollisionNY:Float(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	Local tri:Triangle3df=ib3d_CollisionTriangle(entity,idx,bSourceOnly)
	If tri Then Return tri.getNormal().getY()
EndFunction

Function ib3d_CollisionNZ:Float(entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	Local tri:Triangle3df=ib3d_CollisionTriangle(entity,idx,bSourceOnly)
	If tri Then Return tri.getNormal().getZ()
EndFunction

'-----------------------------------------
' collision type functions
' TODO: note all new ones
'-----------------------------------------

' removes all defined entity collision types
Function ib3d_ClearCollisionTypes()
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	_g_ib3d_collision_manager.clearTypes()
EndFunction

' removes the passed entity collision type
Function ib3d_RemoveCollisionType(collision_type:Int)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	_g_ib3d_collision_manager.removeType(collision_type)
EndFunction

' returns the # of defined entity collision types
Function ib3d_GetCollisionTypeCount:Int()
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.getTypeCount()
EndFunction

' returns the collision type object for the key passed
Function ib3d_GetCollisionType:COLLISIONTYPE(collision_type:Int)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.getType(collision_type)
EndFunction

' returns the collision type object for the index passed
Function ib3d_GetCollisionTypeByIdx:COLLISIONTYPE(type_idx:Int)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.getTypeByIdx(type_idx)
EndFunction

Function ib3d_TypeSetOnCollisionEventHandler(collision_type:COLLISIONTYPE,handler:ONCOLLISIONEVENTHANDLER)
	If Not collision_type Then RuntimeError "Collision type is not valid"
	collision_type.setOnCollisionEventHandler(handler)
EndFunction

Function ib3d_TypeClearEntities(collision_type:COLLISIONTYPE)
	If Not collision_type Then RuntimeError "Collision type is not valid"
	collision_type.clearEntities()			
EndFunction
	
Function ib3d_TypeGetEntityCount:Int(collision_type:COLLISIONTYPE)
	If Not collision_type Then RuntimeError "Collision type is not valid"
	Return collision_type.getEntityCount()
EndFunction

Function ib3d_TypeGetEntity:ib3d_Entity(collision_type:COLLISIONTYPE,idx:Int)
	If Not collision_type Then RuntimeError "Collision type is not valid"
	Return collision_type.getEntity(idx)
EndFunction

Function ib3d_TypeRemoveEntity(collision_type:COLLISIONTYPE,entity:ib3d_Entity,bChildren:Int=False)
	If Not collision_type Then RuntimeError "Collision type is not valid"
	collision_type.removeEntity(entity,bChildren)
EndFunction
																																				
Function ib3d_TypeAddEntity(collision_type:COLLISIONTYPE,entity:ib3d_Entity,bChildren:Int=False)
	If Not collision_type Then RuntimeError "Collision type is not valid"
	collision_type.addEntity(entity,bChildren)
EndFunction	

Function ib3d_TypeContainsEntity:Int(collision_type:COLLISIONTYPE,entity:ib3d_Entity)
	If Not collision_type Then RuntimeError "Collision type is not valid"
	Return collision_type.containsEntity(entity)
EndFunction

Function ib3d_TypeGetType:Int(collision_type:COLLISIONTYPE)
	If Not collision_type Then RuntimeError "Collision type is not valid"
	Return collision_type.getType()
EndFunction

' ----------------------------------------------------
' collision mappings
' ----------------------------------------------------

Function ib3d_AddCustomTypeMapping:COLLISION_MAP(src:Int,dest:Int,det_meth:ICOLLISION_DETECT,response:ICOLLISION_RESPONSE=Null,col_name:String="")
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.addCustomTypeMapping(src,dest,det_meth,response,col_name)
EndFunction
	
Function ib3d_GetTypeMappingCount:Int()	
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.getTypeMappingCount()
EndFunction

Function ib3d_GetTypeMapping:COLLISION_MAP(map_name:String)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.getTypeMapping(map_name)
EndFunction

Function ib3d_GetTypeMappingByIdx:COLLISION_MAP(map_idx:Int)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	Return _g_ib3d_collision_manager.getTypeMappingByIdx(map_idx)
EndFunction

Function ib3d_GetMapDetectionMethodType:Int(map:COLLISION_MAP)
	If Not map Then RuntimeError "Collision map is not valid"
	Return map.getCollisionMethodType()
EndFunction

Function ib3d_GetMapDetectionMethod:ICOLLISION_DETECT(map:COLLISION_MAP)
	If Not map Then RuntimeError "Collision map is not valid"
	Return map.getCollisionMethod()
EndFunction

Function ib3d_SetMapDetectionMethod:Int(map:COLLISION_MAP,detect:ICOLLISION_DETECT)
	If Not map Then RuntimeError "Collision map is not valid"
	If Not detect Then RuntimeError "Detection method is not valid"
	Return map.setCollisionMethod(detect)
EndFunction

Function ib3d_GetMapResponseType:Int(map:COLLISION_MAP)	
	If Not map Then RuntimeError "Collision map is not valid"
	Return map.getCollisionResponseType()	
EndFunction

Function ib3d_GetMapResponse:ICOLLISION_RESPONSE(map:COLLISION_MAP)	
	If Not map Then RuntimeError "Collision map is not valid"
	Return map.getCollisionResponse()	
EndFunction

Function ib3d_SetMapResponse(map:COLLISION_MAP,response:ICOLLISION_RESPONSE)
	If Not map Then RuntimeError "Collision map is not valid"
	map.setCollisionResponse(response)
EndFunction

' TODO: note new function.  useful for tailoring entity collision info to the individual map based on the method of detection.
Function ib3d_SetMapOnSetCollisionMethodHandler(map:COLLISION_MAP,handler:ONSETCOLLISIONMETHODHANDLER)
	If Not map Then RuntimeError "Collision map is not valid"
	map.setOnCollisionMethodHandler(handler)
EndFunction

' returns the # of collisions detected
Function ib3d_MapCheckForCollisions:Int(map:COLLISION_MAP,bRespond:Int=True)
	If Not map Then RuntimeError "Collision map is not valid"
	Return map.checkForCollisions(bRespond)
EndFunction

Function ib3d_MapGetCollisionCount:Int(map:COLLISION_MAP)
	If Not map Then RuntimeError "Collision map is not valid"
	Return map.getCollisionCount()
EndFunction

Function ib3d_MapGetCollision:COLLISION(map:COLLISION_MAP,idx:Int)
	If Not map Then RuntimeError "Collision map is not valid"
	Return map.getCollision(idx)
EndFunction
		
Function ib3d_MapClearCollisions(map:COLLISION_MAP)
	If Not map Then RuntimeError "Collision map is not valid"
	map.clearCollisions()
EndFunction

Function ib3d_MapGetEntityCollisionCount:Int(map:COLLISION_MAP,entity:ib3d_Entity,bSourceOnly:Int=True)
	If Not map Then RuntimeError "Collision map is not valid"
	If Not entity Then RuntimeError "Entity is not valid"
	Return map.getEntityCollisionCount(entity,bSourceOnly)
EndFunction	

Function ib3d_MapGetEntityCollision:COLLISION(map:COLLISION_MAP,entity:ib3d_Entity,idx:Int,bSourceOnly:Int=True)
	If Not map Then RuntimeError "Collision map is not valid"
	If Not entity Then RuntimeError "Entity is not valid"
	Return map.getEntityCollision(entity,idx,bSourceOnly)
EndFunction
	
Function ib3d_MapGetEntityCollisionList:TList(map:COLLISION_MAP,entity:ib3d_Entity,bSourceOnly:Int=True)
	If Not map Then RuntimeError "Collision map is not valid"
	If Not entity Then RuntimeError "Entity is not valid"
	Return map.getEntityCollisionList(entity,bSourceOnly)
EndFunction	

Function ib3d_MapIncludeInCounts(map:COLLISION_MAP,bIncludeInCounts:Int=True)
	If Not map Then RuntimeError "Collision map is not valid"
	map.setIncludeInCounts(bIncludeInCounts)
EndFunction
			
'-----------------------------------------
' Entity collision functions
'-----------------------------------------

Function ib3d_EntityRadius(entity:ib3d_Entity,radius:Float)
	If Not entity Then Throw "Entity does not exist" 
	entity.setCollisionRadius(radius,radius,radius)
EndFunction

Function ib3d_EntityEllipsoid(entity:ib3d_Entity,x:Float,y:Float,z:Float)
	If Not entity Then Throw "Entity does not exist" 
	entity.setCollisionRadius(x,y,z)
EndFunction

Function ib3d_EntityOffset(entity:ib3d_Entity,x:Float,y:Float,z:Float)
	If Not entity Then Throw "Entity does not exist" 
	entity.setCollisionOffset(Vector3df.createFromVals(x,y,z))	
EndFunction

' TODO: note new function
Function ib3d_EntitySetOnCollisionEventHandler(entity:ib3d_Entity,handler:ONCOLLISIONEVENTHANDLER)
	If Not entity Then Throw "Entity does not exist"
	entity.setOnCollisionEventHandler(handler)
EndFunction

Function ib3d_ResetEntity(entity:ib3d_Entity)
	If Not entity Then Throw "Entity does not exist"
	' set last position to current position
	entity.setLastPosition()
EndFunction

Function ib3d_EntityType(entity:ib3d_Entity,collision_type:Int,bRecursive:Int=False)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then Throw "Entity does not exist"
	_g_ib3d_collision_manager.addEntityToType(entity,collision_type,bRecursive)
EndFunction

' TODO: note new function
Function ib3d_EntityRemoveFromType(entity:ib3d_Entity,collision_type:Int,bRecursive:Int=False)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then Throw "Entity does not exist"
	_g_ib3d_collision_manager.removeEntityFromType(entity,collision_type,bRecursive)	
EndFunction

' TODO: note new function
Function ib3d_EntityRemoveFromAllTypes(entity:ib3d_Entity,bRecursive:Int=False)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then Throw "Entity does not exist"
	_g_ib3d_collision_manager.removeEntityFromType(entity,bRecursive)	
EndFunction

Function ib3d_EntityCollided:Int(entity:ib3d_Entity,collision_type:Int)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then Throw "Entity does not exist"
	Return _g_ib3d_collision_manager.collidedWith(entity,collision_type)
EndFunction

' TODO: note new idx param and the return value for not found
Function ib3d_GetEntityType:Int(entity:ib3d_Entity,idx:Int=0)
	If Not _g_ib3d_collision_manager Then RuntimeError "iB3D collision manager has not been initialized"
	If Not entity Then Throw "Entity does not exist"
	Local col:COLLISIONTYPE=_g_ib3d_collision_manager.getEntityTypeByIdx(entity,idx)
	If col Then Return col.getType() Else Return -999
EndFunction

Rem

EntityPickMode 
CollisionSurface:SURFACE - Returns the handle of the surface belonging to the specified entity that was closest to the point of a particular collision.

EndRem

