Public

Global _g_ib3d_font_manager:FONT_MANAGER

Const FS_NONE:Int=0
Const FS_BOLDFONT:Int=1
Const FS_ITALICFONT:Int=2

New FONT_MANAGER

Type FONT_MANAGER

	Field current:FONT=Null
	Field _color:SColor=Null

	Method New()
		_g_ib3d_font_manager=Self
		_color=_SCOLOR(0,255,255,255)
	EndMethod

	Method SetFontColor(red:Int=255,green:Int=255,blue:Int=255,alpha:Int=255)
		_color=_SCOLOR(alpha,red,green,blue)
	EndMethod

	Method SetCurrentFont(fnt:FONT=Null)
		If Not fnt Then fnt=GetDefaultFont()
		current=fnt
	EndMethod
						
	Method GetCurrentFont:FONT()
		If Not current Then SetCurrentFont()
		Return current
	EndMethod									

	Method GetDefaultFont:FONT()
		Local newfont:FONT=Null
		
		If _g_ib3d_engine And _g_ib3d_engine.guienv And _g_ib3d_engine.guienv.IsValid() Then 
			newfont=generate()
			newfont.file="default"
			newfont.font=_g_ib3d_engine.guienv.getBuiltInFont()
			
			If Not newfont.font Or Not newfont.font.IsValid()
				newfont=Null
				DebugLog("Unable to load default font")
			EndIf
		Else
			RuntimeError("iB3D engine has not been initialized")
		EndIf
		
		Return newfont
	EndMethod
															
	Method LoadFont:FONT(file:String,pt_size:Int,bold:Int=False,italics:Int=False)
		Local newfont:FONT=Null
		
		If _g_ib3d_engine And _g_ib3d_engine.guienv And _g_ib3d_engine.guienv.IsValid() Then 
		
			newfont=generate()
			newfont.file=file
			newfont.pt_size=pt_size
			newfont.bold=bold
			newfont.italics=italics
			
			newfont.font=_g_ib3d_engine.guienv.getFont(file)
			
			' make sure the font loaded correctly
			If Not newfont.font Or Not newfont.font.IsValid()
				DebugLog("Unable to load font '"+file+"'")
				newfont=Null
			EndIf
		Else
			RuntimeError("iB3D engine has not been initialized")
		EndIf
		
		Return newfont
	EndMethod

	Method DrawText(text:String,x:Int,y:Int)
		If Not current Then current=GetDefaultFont()
		current.DrawText(text,x,y,-1,-1,_color)
	EndMethod

	Method generate:FONT()
		Return New FONT
	EndMethod
EndType

Type FONT
	
	Field font:IGUIFont=Null
	Field pt_size:Int=0
	Field bold:Int=False
	Field italics:Int=False
	Field file:String

	Method DrawText(text:String,x:Int,y:Int,w:Int=-1,h:Int=-1,color:SColor=Null,bHCenter:Int=False,bVCenter:Int=False)
		Local rect:Recti=Null
		If Not color Then color=_SCOLOR(0,255,255,255)
		
		If w<0 Or h<0 Then 
			Local size:Dimension2di=font.getDimension(text)
			rect=_RECTI(x,y,x+size.getWidth(),y+size.getHeight())
		Else
			rect=_RECTI(x,y,x+w,y+h)
		EndIf

		font.draw(text,rect,color,bHCenter,bVCenter)	
	EndMethod
	
EndType

Rem
bbdoc: Sets the color that is used to draw text
about: <i>Parameters</i>
<ul>
<li>red:Int - red color component.  defaults to 255.</li>
<li>green:Int - green color component.  defaults to 255.</li>
<li>blue:Int - blue color component.  defaults to 255.</li>
<li>alpha:Int - alpha color component.  defaults to 255.</li>
</ul>
Default color is white.
EndRem
Function ib3d_SetFontColor(red:Int=255,green:Int=255,blue:Int=255,alpha:Int=255)
	If Not _g_ib3d_font_manager Then RuntimeError "iB3D font manager has not been initialized"
	_g_ib3d_font_manager.SetFontColor(red,green,blue,alpha)
EndFunction

Rem
bbdoc: Loads an image font
returns: instance of the loaded font
about: <i>Parameters</i>
<ul>
<li>file:String - filename of the font to load</li>
<li>size:Int - PT size of this font</li>
<li>style:Int - style of this font.  defaults to FS_NONE.</li>
</ul>
Font styles are: FS_NONE, FS_BOLDFONT, FS_ITALICFONT
EndRem
Function ib3d_LoadImageFont:FONT(file:String,size:Int,style:Int=FS_NONE)
	If Not _g_ib3d_font_manager Then RuntimeError "iB3D font manager has not been initialized"
	Local bBold:Int=(style & FS_BOLDFONT)
	Local bItalic:Int=(style & FS_ITALICFONT)

	Return _g_ib3d_font_manager.LoadFont(file,size,bBold,bItalic)
EndFunction

Rem
bbdoc: Sets the current font used for ib3d_DrawText commands
about: <i>Parameters</i>
<ul>
<li>fnt:FONT - the new font to use</li>
</ul>
To use the default font pass null or call with no parameter.
EndRem
Function ib3d_SetImageFont(fnt:FONT=Null)
	If Not _g_ib3d_font_manager Then RuntimeError "iB3D font manager has not been initialized"
	_g_ib3d_font_manager.SetCurrentFont(fnt)
EndFunction

Rem
bbdoc: Returns the current font used for ib3d_DrawText commands
returns: instance of the current font
EndRem
Function ib3d_GetImageFont:FONT()
	If Not _g_ib3d_font_manager Then RuntimeError "iB3D font manager has not been initialized"
	Return _g_ib3d_font_manager.GetCurrentFont()
EndFunction

