Rem
' code to determine forward vector
irr::core::vector3df vector = irr::core::vector3df(0,0,1);
irr::core::matrix4 m;
m.setRotationDegrees(node->GetRotation());
m.transformVect(vector);

!!! line3d.getIntersectionWithSphere() to do sphere picking

NOTE: new concept...  ignore bitmask completely.  just get the first one and then 
lookup based on handle.  if not in group, change id and look again.  repeat until find one
in group or no more matches.  be sure to restore old bitmasks.

NOTE: areapick.  use a flat aabbox3df and check each against intersectsWithBox().
EndRem

Public

Global _g_ib3d_pick_manager:PICK_MANAGER

New PICK_MANAGER

Const PICKBIT:Int=28

Rem
bbdoc: PickMode Unpickable (default) 
EndRem
Const PM_UNPICKABLE:Int=0
Rem
bbdoc: PickMode Sphere (not used, will default to PM_BOX)
EndRem
Const PM_SPHERE:Int=1
Rem
bbdoc: PickMode Polygon
EndRem
Const PM_POLYGON:Int=2
Rem
bbdoc: PickMode BoundingBox
EndRem
Const PM_BOX:Int=3

Type PICK_MANAGER
	Global PickableList:TList=Null

	Global _lastPickedTimeMs:Int=0
	Global _lastPickedEntity:ib3d_Entity=Null
	Global _lastPickedTri:Triangle3df=Null
	Global _lastPickedIntersection:Vector3df=Null

	' stores the ray used for the last pick command
	Global _lastPickRay:Line3df=Null
	
	Method New()
		_g_ib3d_pick_manager=Self
		PickableList=New TList
	EndMethod
	
	Method getLastPickRay:Line3df()
		Return _lastPickRay
	EndMethod
	
	Method getLastPickedTimeMs:Int()
		Return _lastPickedTimeMs
	EndMethod
	
	Method getLastPickedEntity:ib3d_Entity()
		Return _lastPickedEntity
	EndMethod
	
	Method getLastPickedTriangle:Triangle3df()
		Return _lastPickedTri
	EndMethod
	
	Method getLastPickedPoint:Vector3df()
		Return _lastPickedIntersection
	EndMethod
	
	Method setLastPickedEntity(entity:ib3d_Entity=Null)
		_lastPickedEntity=Null
	EndMethod
	
	Method setLastPickedInfo(entity:ib3d_Entity,pickTimeMs:Int,intersection:Vector3df,triangle:Triangle3df)
		setLastPickedEntity(entity)
		_lastPickedTimeMs=pickTimeMs
		_lastPickedTri=triangle
		_lastPickedIntersection=intersection
	EndMethod

	Method GetPickBitFlag:Int(bit:Int=PICKBIT,bitFlag:Int=0)
		SetBit(bitFlag,bit)
		Return bitFlag
	EndMethod

	Method SaveIDs(bit:Int=PICKBIT)
		If PickableList Then 
			Local bitFlag:Int=GetPickBitFlag(bit)
			For Local entity:ib3d_Entity=EachIn PickableList
				entity.saveID()
				entity._node.setID(bitFlag) ' set bitFlag
			Next
		EndIf
	EndMethod

	Method RestoreIDs()
		If PickableList Then 
			For Local entity:ib3d_Entity=EachIn PickableList
				entity.restoreID()
			Next
		EndIf
	EndMethod

	Method CameraPick:ib3d_Entity(cam:CAMERA,viewport_x:Int,viewport_y:Int)
		Return RayPick(_g_ib3d_engine.smgr.getSceneCollisionManager().getRayFromScreenCoordinates(_POSITION2DI(viewport_x,viewport_y),cam._cam))
	EndMethod

	' NOTE: radius not supported
	Method LinePick:ib3d_Entity(x:Float,y:Float,z:Float,dx:Float,dy:Float,dz:Float)
		Return RayPick(Line3df.CreateFromVals(x,y,z,dx,dy,dz))
	EndMethod

	Method EntityPick:ib3d_Entity(entity:ib3d_Entity,range:Float=1000.0)
		Local ray_end:Vector3df=entity.GetForwardVector(range)
		Local restore_pm:Int=entity.GetPickMode()		
		Local restore_visible:Int=entity._node.IsVisible()
		entity.SetPickMode(PM_UNPICKABLE)		
		entity._node.SetVisible(False)
		Local picked:ib3d_Entity=RayPick(Line3df.CreateFromVects(entity._node.getPosition(),ray_end))
		entity.SetPickMode(restore_pm)
		entity._node.SetVisible(restore_visible)
		Return picked
	EndMethod

	Method RayPick:ib3d_Entity(ray:Line3df,bit:Int=PICKBIT)
		_lastPickRay=ray

		Local restore_list:TList=New TList				
		Local startMs:Int=MilliSecs()
		Local endMs:Int=0
		Local coll_mgr:ISceneCollisionManager=_g_ib3d_engine.smgr.getSceneCollisionManager()
		Local picked:ib3d_Entity=Null		
		Local tempEntity:ib3d_Entity=Null
		Local node:ISceneNode=Null
		Local bitFlag:Int=GetPickBitFlag(bit)
						
		' store current getID() values
		SaveIDs()

		While True And Not picked
			node=coll_mgr.getSceneNodeFromRayBB(ray,bitFlag)

			' if there was nothing then just exit
			If Not node Then Exit
			
			' irrlicht picking system picks nodes even if bit is set
			If node.getID()=-1 Then 
				node.setID(0)
				If Not restore_list.Contains(node) Then restore_list.AddLast(node)
				Continue
			EndIf
						
			tempEntity=ib3d_Entity.FindEntity(node)
							
			' check type
			If tempEntity And tempEntity.getPickMode()<>PM_UNPICKABLE Then
				Local intersection:Vector3df=_VECTOR3DF()
				Local tri:Triangle3Df=Triangle3DF.Create()
				Local tris:ITriangleSelector=Null
				Local outNode:ISceneNode
			
				' if the found entity is box, then go with it
				If tempEntity.getPickMode()=PM_BOX Then
					endMs=MilliSecs()
					picked=tempEntity
					tris=_g_ib3d_engine.smgr.createTriangleSelectorFromBoundingBox(node)
					coll_mgr.getCollisionPoint(ray,tris,intersection,tri,outNode)
					setLastPickedInfo(picked,(endMs-startMs),intersection,tri)
					picked.OnPicked((endMs-startMs),intersection,tri)
					Exit
				ElseIf tempEntity.getPickMode()=PM_POLYGON ' check the polys
					Local mesh:IMesh=IAnimatedMesh(MESH(tempEntity)._mesh).getMesh(IAnimatedMeshSceneNode(tempEntity._node).getFrameNr())
					
					If mesh And mesh.IsValid() Then
						tris=_g_ib3d_engine.smgr.createTriangleSelector(mesh,node)
						If coll_mgr.getCollisionPoint(ray,tris,intersection,tri,outNode)
							endMs=MilliSecs()
							picked=tempEntity
							setLastPickedInfo(picked,(endMs-startMs),intersection,tri)
							picked.OnPicked((endMs-startMs),intersection,tri)							
							Exit
						Else
							' do not allow this one again
							tempEntity._node.setID(0)
						EndIf					
					EndIf
				EndIf
			Else
				' do not allow this one again
				If tempEntity Then 
					tempEntity._node.setID(0)
					If Not restore_list.Contains(tempEntity._node) Then restore_list.AddLast(tempEntity._node)
				EndIf
			EndIf					
		Wend
		
		' restore getID() values
		RestoreIDs()
		For node=EachIn restore_list
			node.setID(-1)
		Next
		restore_list.Clear()
		
		Return picked
	EndMethod	
EndType

Type ONPICKEDEVENTHANDLER
	Method OnPicked(entity:ib3d_Entity,pickTimeMs:Int,intersection:Vector3df=Null,triangle:Triangle3df=Null) Abstract
	
	Method getPickManager:PICK_MANAGER()
		Return _g_ib3d_pick_manager
	EndMethod
EndType


Rem
bbdoc: Picks the entity positioned at the specified viewport coordinates
returns: the picked entity
about: <i>Parameters</i>
<ul>
<li>camera:CAMERA - camera instance</li>
<li>viewport_x:Int - 2D viewport coordinate</li>
<li>viewport_z:Int - 2D viewport coordinate</li> 
</ul>
Returns the entity picked, or Null if none there. <br><br>
An entity must have its EntityPickMode set to a non-PM_UNPICKABLE (0) value value to be 'pickable'. 
EndRem
Function ib3d_CameraPick:ib3d_Entity(CAMERA:CAMERA,viewport_x:Int,viewport_y:Int)
	If CAMERA=Null Then Throw "Camera does not exist" 
	Return _g_ib3d_pick_manager.CameraPick(CAMERA,viewport_x,viewport_y)
EndFunction

Rem
bbdoc: Returns the nearest entity 'ahead' of the specified entity
returns: the picked entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>range:Float - range to pick.  Defaults to 1000.00</li>
</ul>
An entity must have a non-zero EntityPickMode to be pickable.  
EndRem
Function ib3d_EntityPick:ib3d_Entity(entity:ib3d_Entity,range:Float=1000.00)
	If entity=Null Or Not entity._node.IsValid() Then Throw "Entity does not exist" 
	Return _g_ib3d_pick_manager.EntityPick(entity,range)
EndFunction

Rem
bbdoc: Returns the ray used for the last pick call
returns: the ray as a Line3df
about: 
Returns the ray used for the last pick call or null if a pick command has not been used.
EndRem
Function ib3d_LastPickRay:Line3df()
	Return _g_ib3d_pick_manager.getLastPickRay()
EndFunction

Rem
bbdoc: Returns the first entity between x,y,z to x+dx,y+dy,z+dz
returns: the picked entity
about: <i>Parameters</i>
<ul>
<li>x# - x coordinate of start of line pick </li>
<li>y# - y coordinate of start of line pick </li>
<li>z# - z coordinate of start of line pick </li>
<li>dx# - distance x of line pick </li>
<li>dy# - distance y of line pick </li>
<li>dz# - distance z of line pick </li>
</ul>
Returns the entity picked, or Null if none there. <br><br>
An entity must have its EntityPickMode set to a non-PM_UNPICKABLE (0) value value to be 'pickable'. <br><br>
NOTE: the B3D parameter radius is not supported in iB3D
EndRem
Function ib3d_LinePick:ib3d_Entity(x:Float,y:Float,z:Float,dx:Float,dy:Float,dz:Float)
	Return _g_ib3d_pick_manager.LinePick(x,y,z,dx,dy,dz)
EndFunction

Rem
bbdoc: Sets the pick mode for an entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>mode:Int - type of geometry used for picking.  Defaults to PM_UNPICKABLE.</li>
</ul>
Geometry Types:&nbsp;PM_UNPICKABLE,&nbsp;PM_POLYGON,&nbsp;PM_BOX<br><br>
NOTE: PM_SPHERE is not available in iB3D.  if passed PM_BOX will be used.
EndRem
Function ib3d_EntityPickMode(entity:ib3d_Entity,mode:Int=PM_UNPICKABLE,obscurer:Int=True)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setPickMode(mode,obscurer)
EndFunction

Rem
bbdoc: Sets the OnPicked event handler for an entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>handler:ONPICKEDEVENTHANDLER - instance of an OnPicked event handler</li>
When the entity is the result of a pick command, the handler will be called.
</ul>
EndRem
Function ib3d_EntitySetOnPickedEventHandler(entity:ib3d_Entity,handler:ONPICKEDEVENTHANDLER)
	If Not entity Then Throw "Entity does not exist"
	entity.setOnPickedEventHandler(handler)
EndFunction

Rem
bbdoc: Returns the entity 'picked' by the most recently executed Pick command
returns: The last picked entity
about:
Last pick command might have been CameraPick, EntityPick or LinePick. <br><br>
Returns Null if no entity was picked. 
EndRem
Function ib3d_PickedEntity:ib3d_Entity()
	Return _g_ib3d_pick_manager.getLastPickedEntity()
EndFunction

Rem
bbdoc: Returns the time taken to calculate the most recently executed Pick command 
returns: Time taken in milliseconds
about:
Last pick command might have been CameraPick, EntityPick or LinePick.
EndRem
Function ib3d_PickedTime:Int()
	Return _g_ib3d_pick_manager.getLastPickedTimeMs()
EndFunction

Rem
bbdoc: Returns the triangle that was 'picked' by the most recently executed Pick command
returns: Triangle picked as Triangle3df
about:
Last pick command might have been CameraPick, EntityPick Or LinePick.<br><br>
NOTE: This is different than B3D.
EndRem
Function ib3d_PickedTriangle:Triangle3df()
	Return _g_ib3d_pick_manager.getLastPickedTriangle()
EndFunction

Rem
bbdoc: Returns the world coordinate of the most recently executed Pick command
returns: Coordinate as a Vector3df
about:
Last pick command might have been CameraPick, EntityPick or LinePick.<br><br>
The coordinate represents the exact point of where something was picked. 
EndRem
Function ib3d_PickedPoint:Vector3df()
	Return _g_ib3d_pick_manager.getLastPickedPoint()
EndFunction

Rem
bbdoc: Returns the world X coordinate of the most recently executed Pick command
returns: X coordinate value
about:
Last pick command might have been CameraPick, EntityPick or LinePick.<br><br>
The coordinate represents the exact point of where something was picked. 
EndRem
Function ib3d_PickedX:Float()
	If _g_ib3d_pick_manager.getLastPickedPoint() Then
		Return _g_ib3d_pick_manager.getLastPickedPoint().getX()
	Else
		Return 0
	EndIf
EndFunction

Rem
bbdoc: Returns the world Y coordinate of the most recently executed Pick command
returns: Y coordinate value
about:
Last pick command might have been CameraPick, EntityPick or LinePick.<br><br>
The coordinate represents the exact point of where something was picked. 
EndRem
Function ib3d_PickedY:Float()
	If _g_ib3d_pick_manager.getLastPickedPoint() Then
		Return _g_ib3d_pick_manager.getLastPickedPoint().getY()
	Else
		Return 0
	EndIf
EndFunction

Rem
bbdoc: Returns the world Z coordinate of the most recently executed Pick command
returns: Z coordinate value
about:
Last pick command might have been CameraPick, EntityPick or LinePick.<br><br>
The coordinate represents the exact point of where something was picked. 
EndRem
Function ib3d_PickedZ:Float()
	If _g_ib3d_pick_manager.getLastPickedPoint() Then
		Return _g_ib3d_pick_manager.getLastPickedPoint().getZ()
	Else
		Return 0
	EndIf
EndFunction

Rem
bbdoc: Returns the X component of the normal of the most recently executed Pick command
returns: X normal value
about:
Last pick command might have been CameraPick, EntityPick or LinePick.
EndRem
Function ib3d_PickedNX:Float()
	If _g_ib3d_pick_manager.getLastPickedTriangle() Then
		Return _g_ib3d_pick_manager.getLastPickedTriangle().getNormal().getX()
	Else
		Return 0
	EndIf
EndFunction

Rem
bbdoc: Returns the Y component of the normal of the most recently executed Pick command
returns: Y normal value
about:
Last pick command might have been CameraPick, EntityPick or LinePick.
EndRem
Function ib3d_PickedNY:Float()
	If _g_ib3d_pick_manager.getLastPickedTriangle() Then
		Return _g_ib3d_pick_manager.getLastPickedTriangle().getNormal().getY()
	Else
		Return 0
	EndIf
EndFunction

Rem
bbdoc: Returns the Z component of the normal of the most recently executed Pick command
returns: Z normal value
about:
Last pick command might have been CameraPick, EntityPick or LinePick.
EndRem
Function ib3d_PickedNZ:Float()
	If _g_ib3d_pick_manager.getLastPickedTriangle() Then
		Return _g_ib3d_pick_manager.getLastPickedTriangle().getNormal().getZ()
	Else
		Return 0
	EndIf
EndFunction

