' ------------------------------------------------------------------------------------
'  irr::scene::ISceneNodeAnimatorCollisionResponse wrappers (based on ISceneNodeAnimator)
' ------------------------------------------------------------------------------------
Public

' ------------------------------------------------------------------------------------
' ICollisionCallback
' ------------------------------------------------------------------------------------

Type ICollisionCallback Extends IReferenceCounted
	
	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------
	
	Method onCollision(animator:ISceneNodeAnimatorCollisionResponse) Abstract
	
	' --------------------------------------------------------------------------------
	' base routines
	' --------------------------------------------------------------------------------

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_BMXICollisionCallback_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_BMXICollisionCallback_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_BMXICollisionCallback_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod

	' --------------------------------------------------------------------------------
	' creation routines
	' --------------------------------------------------------------------------------

	' this should only be called on an extended type
	Function create:ICollisionCallback(pGenFunc:Byte Ptr) Final
		Return createFromHandle(IrrScene_BMXICollisionCallback_new(),True,pGenFunc)
	EndFunction

	Function createFromHandle:ICollisionCallback(pHandle:Int,bMustDel:Int,pGenFunc:Byte Ptr) Final
		If Not pHandle Then Return Null
		Local oRetVal:ICollisionCallback=Null
		
		If pGenFunc
			' use the funtion pointer to generate
			Local generate:ICollisionCallback()=pGenFunc
		
			oRetVal=generate()
		EndIf

		' turn binding on if this is the first and there was a gen func passed in
		oRetVal.bIsBound=bMustDel And pGenFunc
		
		oRetVal.attach(pHandle,bMustDel)
		
		Return oRetVal
	EndFunction

	' --------------------------------------------------------------------------------
	' bindings
	' --------------------------------------------------------------------------------

	Method bind(rec:Int) Final
		Super.bind(rec)
	EndMethod

	Method getBinderHandle:Int()
		Return IrrScene_BMXICollisionCallback_getBinder(handle)
	EndMethod

	Function BoundOnCollision(binding:Int, animator:Int)
		' get the object whose node is firing
		Local tempobj:ICollisionCallback = ICollisionCallback(translate(binding))
		
		If tempobj<>Null
			Local obj:ISceneNodeAnimatorCollisionResponse = ISceneNodeAnimatorCollisionResponse.createFromHandle(animator, False)
							
  			' fire the event
	  		tempobj.onCollision(obj)
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

EndType

Type ISceneNodeAnimatorCollisionResponse Extends ISceneNodeAnimator Final

	Function createFromHandle:ISceneNodeAnimatorCollisionResponse(pHandle:Int,bMustDel:Int=False,pGenFunc:Byte Ptr=Null)
		If Not pHandle Then Return Null
		Local oRetVal:ISceneNodeAnimatorCollisionResponse=New ISceneNodeAnimatorCollisionResponse
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction
		
	Method getEllipsoidRadius:Vector3df()
		Return Vector3df.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getEllipsoidRadius(handle))
	EndMethod
	
	Method getEllipsoidTranslation:Vector3df()	
		Return Vector3df.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getEllipsoidTranslation(handle))
	EndMethod
	
	Method getGravity:Vector3df()
		Return Vector3df.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getGravity(handle))
	EndMethod
	
	Method getWorld:ITriangleSelector()
		'  do Not Delete this pointer
		Return ITriangleSelector.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getWorld(handle),False)
	EndMethod
	
	Method isFalling:Int()
		Return IrrScene_ISceneNodeAnimatorCollisionResponse_isFalling(handle)
	EndMethod
	
	Method setEllipsoidRadius(radius:Vector3df)
		IrrScene_ISceneNodeAnimatorCollisionResponse_setEllipsoidRadius(handle,radius.handle)
	EndMethod

	Method setEllipsoidTranslation(translation:Vector3df)
		IrrScene_ISceneNodeAnimatorCollisionResponse_setEllipsoidTranslation(handle,translation.handle)
	EndMethod
	
	Method setGravity(gravity:Vector3df)
		IrrScene_ISceneNodeAnimatorCollisionResponse_setGravity(handle,gravity.handle)
	EndMethod
	
	Method setWorld(newWorld:ITriangleSelector)
		IrrScene_ISceneNodeAnimatorCollisionResponse_setWorld(handle,newWorld.handle)
	EndMethod

	Method jump(jumpSpeed:Float)
		IrrScene_ISceneNodeAnimatorCollisionResponse_jump(handle, jumpSpeed) 
	EndMethod

	Method setAnimateTarget(bEnable:Int)
		IrrScene_ISceneNodeAnimatorCollisionResponse_setAnimateTarget(handle, bEnable)
	EndMethod
	
	Method getAnimateTarget:Int()
		Return IrrScene_ISceneNodeAnimatorCollisionResponse_getAnimateTarget(handle)
	EndMethod
	
	Method setTargetNode(node:ISceneNode)
		IrrScene_ISceneNodeAnimatorCollisionResponse_setTargetNode(handle, _HANDLE(node))
	EndMethod
	
	Method getTargetNode:ISceneNode()
		Return ISceneNode.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getTargetNode(handle), False)
	EndMethod
	
	Method collisionOccurred:Int()
		Return IrrScene_ISceneNodeAnimatorCollisionResponse_collisionOccurred(handle)
	EndMethod
	
	Method getCollisionPoint:Vector3df()
		Return Vector3df.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getCollisionPoint(handle), True)
	EndMethod
	
	Method getCollisionTriangle:Triangle3df()
		Return Triangle3df.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getCollisionTriangle(handle), True)
	EndMethod
	
	Method getCollisionResultPosition:Vector3df()
		Return Vector3df.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getCollisionResultPosition(handle), True)
	EndMethod
	
	Method getCollisionNode:ISceneNode()
		Return ISceneNode.createFromHandle(IrrScene_ISceneNodeAnimatorCollisionResponse_getCollisionNode(handle), False)
	EndMethod
	
	Method setCollisionCallback(callback:ICollisionCallback)
		IrrScene_ISceneNodeAnimatorCollisionResponse_setCollisionCallback(handle, _HANDLE(callback))
	EndMethod

EndType

