
Public

Type IImageLoader Extends IReferenceCounted
	
	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------

	Method loadImage:IImage(file:IReadFile)
		If Not bIsBound Return IImage.createFromHandle(IrrVideo_IImageLoader_loadImage(handle, _HANDLE(file)), True)
	EndMethod

	Method isALoadableFileFormat:Int(file:IReadFile)
		If Not bIsBound Return IrrVideo_IImageLoader_isALoadableFileFormat(handle, _HANDLE(file))
	EndMethod

	Method isALoadableFileExtension:Int(fileName:String)
		If Not bIsBound Return IrrVideo_IImageLoader_isALoadableFileExtension(handle, fileName)
	EndMethod
	
	' --------------------------------------------------------------------------------
	' creation routines
	' --------------------------------------------------------------------------------

	' this should only be called on an extended type
	Function create:IImageLoader(pGenFunc:Byte Ptr) Final
		Return createFromHandle(IrrVideo_BMXIImageLoader_new(),True,pGenFunc)
	EndFunction

	Function createFromHandle:IImageLoader(pHandle:Int,bMustDel:Int,pGenFunc:Byte Ptr = Null) Final
		If Not pHandle Then Return Null
		Local oRetVal:IImageLoader=Null
		
		If pGenFunc Then
			' use the funtion pointer to generate
			Local generate:IImageLoader()=pGenFunc
		
			oRetVal=generate()
		Else 
			oRetVal = New IImageLoader
		EndIf
		
		' turn binding on if this is the first and there was a gen func passed in
		oRetVal.bIsBound = (bMustDel And pGenFunc)
		
		oRetVal.attach(pHandle, bMustDel)
		
		Return oRetVal
	EndFunction

	' --------------------------------------------------------------------------------
	' bindings
	' --------------------------------------------------------------------------------

	Method bind(rec:Int) Final
		Super.bind(rec)
		IrrVideo_BMXIImageLoader_bindIsALoadableFileExtension( ..
			translate(rec).handle, ..
			OnIsALoadableFileExtension)
		IrrVideo_BMXIImageLoader_bindIsALoadableFileFormat( ..
			translate(rec).handle, ..
			OnIsALoadableFileFormat)
		IrrVideo_BMXIImageLoader_bindLoadImage( ..
			translate(rec).handle, ..
			OnLoadImage)
	EndMethod

	Method getBinderHandle:Int()
		Return IrrVideo_BMXIImageLoader_getBinder(handle)
	EndMethod

	Function OnLoadImage:Int(binding:Int,pIReadFile:Int)
		' get the object whose node is firing
		Local tempobj:IImageLoader=IImageLoader(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Local retval:IImage=tempobj.loadImage(IReadFile.createFromHandle(pIReadFile,False))
	  		If Not retval Or Not retval.IsValid() Then Return 0 Else Return retval.handle
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

	Function OnIsALoadableFileFormat:Int(binding:Int,pIReadFile:Int)
		' get the object whose node is firing
		Local tempobj:IImageLoader=IImageLoader(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.isALoadableFileFormat(IReadFile.createFromHandle(pIReadFile,False))
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

	Function OnIsALoadableFileExtension:Int(binding:Int,fileName:Byte Ptr)
		' get the object whose node is firing
		Local tempobj:IImageLoader=IImageLoader(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.isALoadableFileExtension(String.FromCString(fileName))
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction
	
EndType
